-- Do not use this file except in compliance with the License. You may
-- obtain a copy of the License at http://www.microcore.org/License/
-- Software distributed under the License is distributed on an "AS IS" basis,
-- WITHOUT WARRANTY OF ANY KIND, either express or implied.
-- See the License for the specific language governing rights and limitations
-- under the License.
--
-- The Original Code is: MicroCore DEBUG_BENCH.VHD 1.10
--
-- The Initial Developer of the Original Code is Klaus.Schleisiek AT hamburg.de.
--
-- Introduction  see: information/uCore_Introduction.pdf
-- Tools         see: information/uCore_Implementation.pdf
-- Versions      see: uCore_Versions.txt
-- License       see: information/uCore_License.pdf
-- Conventions   see: information/uCore_Conventions.pdf
-- Contributors  see: information/uCore_Contributors.pdf
-- Philosophy    see: information/uCore_Philosophy.pdf
--
-- Test bench to be used with internal data RAM and external RAM serving as 
-- program ROM.
-- After reset, the object code will be loaded via the umbilical and executed.

LIBRARY IEEE;
USE     IEEE.std_logic_1164.ALL;
USE     work.functions.ALL;
USE     work.constants.ALL;
                                
ENTITY bench IS
END bench;

ARCHITECTURE testbench OF bench IS

COMPONENT core
PORT (reset       : IN    std_logic;
      clk         : IN    std_logic;
      clk_en      : IN    std_logic;
      delay       : INOUT std_logic; -- unused pad, serves to generate tri_en signal
-- io interface
      iosel       : OUT   std_logic;
      iotrap      : IN    std_logic;
      iowrite     : OUT   std_logic;
      ioaddr      : OUT   data_addr;
      iodout      : OUT   data_bus;
      iodin       : IN    data_bus;
-- Centronics port for debugging
      init_n      : IN    std_logic;                     -- NOT init
      busy        : OUT   std_logic;                     --     busy
      autofd      : IN    std_logic;                     --     in_atn
      strobe_n    : IN    std_logic;                     -- NOT in_str
      ack_n       : OUT   std_logic;                     -- NOT in_ack
      data        : IN    std_logic_vector(7 DOWNTO 0);  --     in_data
      error_n     : OUT   std_logic;                     -- NOT out_atn
      slct        : OUT   std_logic;                     --     out_str
      slctin_n    : IN    std_logic;                     -- NOT out_ack
      pe          : OUT   std_logic_vector(0 DOWNTO 0);  --     out_data
-- Program memory - RAM for debugging
      romwr       : OUT   std_logic;
      romdin      : OUT   inst_bus;
      romaddr     : OUT   prog_addr;
      instruction : IN    inst_bus;
-- Data memory
--      ramwr       : OUT   std_logic;
--      ramaddr     : OUT   data_addr;
--      ramdin      : OUT   data_bus;
--      ramdout     : IN    data_bus;
      int_in      : IN    int_bus);
END COMPONENT;

COMPONENT data_ram
PORT (dram : INOUT dram_port);
END COMPONENT;

COMPONENT code_rom
PORT (rom : INOUT rom_port);
END COMPONENT;

COMPONENT debug_rom
PORT (debugROM : INOUT debugrom_port);
END COMPONENT;

SIGNAL reset      : std_logic; 
SIGNAL clk        : std_logic;
SIGNAL clk_en     : std_logic; 
SIGNAL iotrap     : std_logic;  
SIGNAL interrupt  : int_bus;

SIGNAL iosel      : std_logic;
SIGNAL iowrite    : std_logic;
SIGNAL ioaddr     : data_addr;
SIGNAL iodout     : data_bus;
SIGNAL iodin      : data_bus;
SIGNAL ioreg      : data_bus; 

SIGNAL init_n     : std_logic; 
SIGNAL busy       : std_logic; 
SIGNAL strobe_n   : std_logic; 
SIGNAL ack_n      : std_logic; 
SIGNAL data       : std_logic_vector(7 DOWNTO 0);
SIGNAL error_n    : std_logic; 
SIGNAL autofd     : std_logic; 
SIGNAL slct       : std_logic; 
SIGNAL pe         : std_logic_vector(0 DOWNTO 0); 
SIGNAL slctin_n   : std_logic; 

SIGNAL debugROM   : debugrom_port; 
SIGNAL rom        : rom_port; 
--SIGNAL dram       : dram_port; 

CONSTANT IOPORT1  : data_addr := to_vec(-5, data_addr_width); -- for testbench
CONSTANT IOPORT2  : data_addr := to_vec(-6, data_addr_width); -- for testbench

CONSTANT cycle    : TIME := 100 ns;

BEGIN

debugcore: core PORT 
MAP ( reset       => reset,
      clk         => clk,
      clk_en      => clk_en,

      iosel       => iosel,
      iotrap      => iotrap,
      iowrite     => iowrite,
      ioaddr      => ioaddr,
      iodout      => iodout,
      iodin       => iodin,

      init_n      => init_n,
      busy        => busy,
      autofd      => autofd,
      strobe_n    => strobe_n,
      ack_n       => ack_n,
      data        => data,
      error_n     => error_n,
      slct        => slct,
      slctin_n    => slctin_n,
      pe          => pe,

      romwr       => debugROM.write,
      romaddr     => debugROM.addr,
      romdin      => debugROM.din,
      instruction => debugROM.dout,

--      ramwr       => dram.write,
--      ramaddr     => dram.addr,
--      ramdin      => dram.din,
--      ramdout     => dram.dout,

      int_in      => interrupt);

program: debug_rom PORT MAP(debugROM => debugROM);
debugROM.clk <= clk;

download: code_rom PORT MAP(rom => rom);

--data_memory: data_ram PORT MAP (dram => dram);
--dram.clk <= clk;

interrupt(0) <= busy;

reset_core: PROCESS

PROCEDURE send2core (number : IN data_bus) IS
BEGIN
  WAIT FOR cycle/5;
  WHILE  busy='1'  LOOP  WAIT FOR cycle/5; END LOOP;
  autofd <= '1';
  data <= "0000" & number(11 DOWNTO 8);
  WAIT FOR cycle/5;
  strobe_n <= '0';
  WAIT UNTIL ack_n='0';
  WAIT FOR cycle/5;
  strobe_n <= '1';
  WAIT UNTIL ack_n='1';
  data <= number(7 DOWNTO 0);
  WAIT FOR cycle/5;
  strobe_n <= '0';
  WAIT UNTIL ack_n='0';
  WAIT FOR cycle/5;
  strobe_n <= '1';
  WAIT UNTIL ack_n='1';
  autofd <= '0';
  WAIT FOR cycle/5;
  strobe_n <= '0';
  WAIT UNTIL ack_n='0';
  WAIT FOR cycle/5;
  strobe_n <= '1';
  WAIT UNTIL ack_n='1';
  data <= "ZZZZZZZZ";
END send2core;

BEGIN
   reset        <= '1';
   clk_en       <= '1';
   interrupt(1) <= '0';
   iotrap       <= '0';
   init_n       <= '1';
   strobe_n     <= '1';
   data         <= "ZZZZZZZZ";
   autofd       <= '0';
   rom.addr     <= (OTHERS => '0');
   WAIT FOR cycle + cycle/5;
   reset        <= '0';
   WAIT FOR cycle/5;

   init_n       <= '0';
   WAIT UNTIL busy='1';
   WAIT FOR cycle/5;
   strobe_n     <= '0';
   WAIT FOR cycle*3;
   WAIT FOR cycle/5;
   strobe_n     <= '1';
   WAIT FOR cycle/5;
   init_n       <= '1';
   WAIT UNTIL busy='0';

FOR  i IN 0 TO (2 ** prog_addr_width)-1  LOOP
   rom.addr     <= to_vec(i, prog_addr_width);
   WAIT FOR cycle/5;
   strobe_n     <= '0';
   WAIT FOR cycle;
   data         <= rom.data;
   WAIT UNTIL ack_n='0';
   WAIT FOR cycle/5;
   strobe_n     <= '1';
   WAIT UNTIL busy='0';
END LOOP;

   data         <= (OTHERS => 'Z');

   WAIT FOR 4 us;
   send2core("010111110101");
   send2core("010111110101");
   send2core("000000001010");
   send2core("000000001010");
   WAIT;
END PROCESS reset_core;

testio: PROCESS(reset, clk)
BEGIN
  IF  reset='1'  THEN
     ioreg <= (OTHERS => '0');
  ELSIF  rising_edge(clk)  THEN
     IF  clk_en='1'  THEN
       IF  iosel='1' AND iowrite='1' AND ioaddr=IOPORT1  THEN
           ioreg <= iodout;
       END IF;
     END IF;
  END IF;
END PROCESS testio;

iodin <= ioreg WHEN  ioaddr=IOPORT1  ELSE  to_vec(85, data_width);

clock: PROCESS
BEGIN
  LOOP
    clk <= '1';
    WAIT FOR cycle/2;
    clk <= '0';
    WAIT FOR cycle/2;
  END LOOP;
END PROCESS clock;

from_target: PROCESS
BEGIN
  slctin_n <= '1';
  LOOP
    WAIT UNTIL error_n='0';
    WHILE  error_n='0'  LOOP
      WHILE slct='0' LOOP 
        WAIT FOR cycle/5;
      END LOOP;
      slctin_n <= '0';
      WAIT UNTIL slct='0';
      WAIT FOR cycle/5;
      slctin_n <= '1';
    END LOOP;
    WAIT FOR cycle/5;
    WHILE slct='0' LOOP 
      WAIT FOR cycle/5;
    END LOOP;
    slctin_n <= '0';
    WAIT UNTIL slct='0';
    WAIT FOR cycle/5;
    slctin_n <= '1';
  END LOOP;
END PROCESS from_target;


END testbench;
