-- Do not use this file except in compliance with the License. You may
-- obtain a copy of the License at http://www.microcore.org/License/
-- Software distributed under the License is distributed on an "AS IS" basis,
-- WITHOUT WARRANTY OF ANY KIND, either express or implied.
-- See the License for the specific language governing rights and limitations
-- under the License.
--
-- The Original Code is: MicroCore DEBUG_CENTRONICS.VHD 1.10
--
-- The Initial Developer of the Original Code is Klaus.Schleisiek AT hamburg.de.
--
-- Introduction  see: information/uCore_Introduction.pdf
-- Tools         see: information/uCore_Implementation.pdf
-- Versions      see: uCore_Versions.txt
-- License       see: information/uCore_License.pdf
-- Conventions   see: information/uCore_Conventions.pdf
-- Contributors  see: information/uCore_Contributors.pdf
-- Philosophy    see: information/uCore_Philosophy.pdf
--
-- Debug interface using the Centronics port.
-- It consists of a program memory loader and an IN and 
-- OUT register which can be written and read by the 
-- host asynchronously and read and written by the core 
-- as DEBUG_REGISTER.

LIBRARY IEEE;
USE     IEEE.std_logic_1164.ALL;
USE     IEEE.std_logic_signed.ALL;
USE     work.functions.ALL;
USE     work.constants.ALL;

ENTITY debugger IS
PORT (reset      : IN    std_logic;
      clk        : IN    std_logic;
      clk_en     : IN    std_logic;

      rom        : INOUT rom_port;
      ioBus      : INOUT io_port;
      debug      : INOUT umbilical;
      debugROM   : INOUT debugrom_port;

      in_reg     : OUT   data_bus;
      core_reset : OUT   std_logic;       
      out_trap   : OUT   std_logic; 
      rom_load   : OUT   std_logic);
END debugger;

ARCHITECTURE rtl OF debugger IS

SIGNAL init       : std_logic;    -- host initiates program loading
SIGNAL in_str     : std_logic;    -- host signals valid data
SIGNAL in_ack     : std_logic;    -- target captures data on rising_edge
SIGNAL in_data    : std_logic_vector(in_width-1 DOWNTO 0); -- data from host
SIGNAL in_rd      : std_logic;    -- target reads IN_REG
SIGNAL in_reg_i   : data_bus;     
SIGNAL in_atn     : std_logic;    -- host transmits one word to IN
SIGNAL in_busy    : std_logic;    -- target can not receive

SIGNAL program    : prog_addr;    -- rom.addr
SIGNAL addr_ctr   : prog_addr;    -- incremental debugROM address for loading
SIGNAL write      : std_logic;    -- write debugROM

SIGNAL out_atn    : std_logic;    -- target transfers OUT to host
SIGNAL out_str    : std_logic;    -- target signals valid data
SIGNAL out_ack    : std_logic;    -- host captures data on rising_edge
SIGNAL out_reg    : data_bus;     -- out_data
SIGNAL out_wr     : std_logic;    -- target writes OUT_REG
SIGNAL out_ctr    : std_logic_vector(log2(data_width)-1 DOWNTO 0); -- #bits to transfer to host

TYPE in_states IS (idle, loading, receiving, full);
SIGNAL in_state  : in_states;

TYPE out_states IS (idle, sending, ready);
SIGNAL out_state : out_states;
 
BEGIN

init    <= debug.init;     -- for the synthesizer
in_str  <= debug.in_str;   -- for the synthesizer
in_data <= debug.in_data;  -- for the synthesizer
in_atn  <= debug.in_atn;   -- for the synthesizer

core_reset <= reset OR (init AND in_str);

-----------------------------------------------------------------------
-- emulate program ROM from the debugROM
-----------------------------------------------------------------------

program  <= rom.addr;
rom.data <= debugROM.dout;

-----------------------------------------------------------------------
-- loading the program memory
-- and listening to the host  
-----------------------------------------------------------------------

in_rd <= NOT ioBus.write WHEN  ioBus.sel='1' AND ioBus.addr=DEBUG_REGISTER  ELSE '0';

loadROM: PROCESS(clk, reset)
BEGIN
  IF  reset='1'  THEN
     in_state <= idle;
     addr_ctr <= (OTHERS => '0');
     in_reg_i <= (OTHERS => '0');
       in_ack <= '0';
  ELSIF  rising_edge(clk)  THEN
    in_ack <= in_str;
    IF  init='1'  THEN
      addr_ctr <= (OTHERS => '0');
        in_ack <= '0';
      in_state <= loading;
    ELSE  
      CASE in_state IS
      WHEN loading => 
          IF  (in_str AND NOT in_ack)='1'  THEN
            addr_ctr <= addr_ctr + 1;
            IF  addr_ctr = -1  THEN
               in_state <= idle;
            END IF;
          END IF;
      WHEN receiving =>
          IF  in_atn='0'  THEN
            in_state <= full;
          ELSIF  (in_str AND NOT in_ack)='1'  THEN
            in_reg_i <= in_reg_i(data_width-in_width-1 DOWNTO 0) & in_data;
          END IF;
      WHEN full => 
          IF  (in_rd AND clk_en)='1'  THEN
            in_state <= idle;
          END IF;
      WHEN idle =>
          in_ack <= '0'; 
          IF  in_atn='1'  THEN
            in_reg_i <= (OTHERS => '0');
            in_state <= receiving;
          END IF;
      WHEN OTHERS => in_state <= idle;
      END CASE;
    END IF; 
  END IF;
END PROCESS loadROM;

in_reg  <= in_reg_i;

in_busy <= '1' WHEN  in_state=full  ELSE '0';

debug.in_ack <= in_ack AND in_str;
debug.busy   <= init OR (in_ack AND in_str) OR in_busy;

write    <= '1' WHEN  in_state=loading  ELSE '0';
rom_load <= write OR init;

debugROM.clk   <= clk;
debugROM.din   <= in_data;
debugROM.write <= write;
debugROM.addr  <= program WHEN  write='0'  ELSE addr_ctr;

-----------------------------------------------------------------------
-- talking to the host 
-----------------------------------------------------------------------

out_ack <= debug.out_ack;     -- for the synthesizer

out_wr  <= ioBus.write WHEN  ioBus.sel='1' AND ioBus.addr=DEBUG_REGISTER  ELSE '0';

debug_out: PROCESS (clk, reset)
BEGIN
  IF  reset='1'  THEN
       out_str <= '0';
       out_reg <= (OTHERS => '0');
       out_ctr <= (OTHERS => '1');
     out_state <= idle;
  ELSIF  rising_edge(clk)  THEN
    out_str <= NOT out_ack;
    CASE out_state IS
    WHEN idle =>
        IF  out_wr='1' AND clk_en='1'  THEN   -- send a new word
            out_reg <= ioBus.dout;
            out_ctr <= to_vec(data_width-1, log2(data_width));
          out_state <= sending;
        ELSE
          out_str <= '0';
        END IF;
    WHEN sending =>
        IF  (out_ack AND out_str)='1'  THEN  -- shift next bit on out_ack
          out_reg <= '0' & out_reg(out_reg'high DOWNTO 1);
          out_ctr <= out_ctr - 1;
          IF  out_ctr=0  THEN  out_state <= ready;  END IF;
        END IF;
    WHEN ready => 
        IF  (out_ack AND out_str)='1'  THEN
          out_state <= idle;
        END IF;
    WHEN OTHERS => out_state <= idle;
    END CASE;
  END IF;
END PROCESS debug_out;

out_atn <= '1' WHEN  out_state=sending  ELSE '0';

debug.out_str     <= out_str;
debug.out_atn     <= out_atn;
debug.out_data(0) <= out_reg(0);

out_trap   <= '1' WHEN  out_wr='1' AND (out_state=sending OR out_state=ready)  ELSE '0';

ioBus.trap <= 'Z';


END rtl;
