-- Do not use this file except in compliance with the License. You may
-- obtain a copy of the License at http://www.microcore.org/License/
-- Software distributed under the License is distributed on an "AS IS" basis,
-- WITHOUT WARRANTY OF ANY KIND, either express or implied.
-- See the License for the specific language governing rights and limitations
-- under the License.
--
-- The Original Code is: MicroCore DEBUG_CORE.VHD 1.10
--
-- The Initial Developer of the Original Code is Klaus.Schleisiek AT hamburg.de.
--
-- Introduction  see: information/uCore_Introduction.pdf
-- Tools         see: information/uCore_Implementation.pdf
-- Versions      see: uCore_Versions.txt
-- License       see: information/uCore_License.pdf
-- Conventions   see: information/uCore_Conventions.pdf
-- Contributors  see: information/uCore_Contributors.pdf
-- Philosophy    see: information/uCore_Philosophy.pdf
--
-- Top level entity with external RAM serving as program ROM and internal
-- asynchronous data/return-stack RAM. 
-- This version uses a centronics interface as umbilical.
-- This file should be edited for technology specific additions like e.g.
-- pad assignments.

LIBRARY IEEE;
USE     IEEE.std_logic_1164.ALL;
USE     work.functions.ALL;
USE     work.constants.ALL;

ENTITY core IS
PORT (reset       : IN    std_logic;
      clk         : IN    std_logic;
      clk_en      : IN    std_logic;
      delay       : INOUT std_logic; -- unused pad, serves to generate tri_en signal
-- io interface
      iosel       : OUT   std_logic;
      iotrap      : IN    std_logic;
      iowrite     : OUT   std_logic;
      ioaddr      : OUT   data_addr;
      iodout      : OUT   data_bus;
      iodin       : IN    data_bus;
-- Centronics port for debugging
      init_n      : IN    std_logic;                     -- NOT init
      busy        : OUT   std_logic;                     --     busy
      autofd      : IN    std_logic;                     --     in_atn
      strobe_n    : IN    std_logic;                     -- NOT in_str
      ack_n       : OUT   std_logic;                     -- NOT in_ack
      data        : IN    std_logic_vector(7 DOWNTO 0);  --     in_data
      error_n     : OUT   std_logic;                     -- NOT out_atn
      slct        : OUT   std_logic;                     --     out_str
      slctin_n    : IN    std_logic;                     -- NOT out_ack
      pe          : OUT   std_logic_vector(0 DOWNTO 0);  --     out_data
-- Program memory - RAM for debugging
      romwr       : OUT   std_logic;
      romdin      : OUT   inst_bus;
      romaddr     : OUT   prog_addr;
      instruction : IN    inst_bus;
-- Data memory
--      ramwr       : OUT   std_logic;
--      ramaddr     : OUT   data_addr;
--      ramdin      : OUT   data_bus;
--      ramdout     : IN    data_bus;
      int_in      : IN    int_bus);
END core;

ARCHITECTURE technology OF core IS

COMPONENT microcore
PORT (reset     : IN    std_logic; 
      clk       : IN    std_logic; 
      clk_en    : IN    std_logic;
      tri_en    : IN    std_logic;  
      int_in    : IN    int_bus;
      dram      : INOUT dram_port;
      rom       : INOUT rom_port;
      ioBus     : INOUT io_port);
END COMPONENT;

COMPONENT data_ram
PORT (dram : INOUT dram_port);
END COMPONENT;

COMPONENT debugger 
PORT (reset      : IN    std_logic;
      clk        : IN    std_logic;
      clk_en     : IN    std_logic;

      rom        : INOUT rom_port;
      ioBus      : INOUT io_port;
      debug      : INOUT umbilical;
      debugROM   : INOUT debugrom_port;
      
      in_reg     : OUT   data_bus;
      core_reset : OUT   std_logic;       
      out_trap   : OUT   std_logic; 
      rom_load   : OUT   std_logic);
END COMPONENT;

COMPONENT synchronise
PORT (reset    : IN    std_logic;
      clk      : IN    std_logic;
      async    : IN    std_logic; 
      sync     : OUT   std_logic);
END COMPONENT;

COMPONENT synchronise_n
PORT (reset    : IN    std_logic;
      clk      : IN    std_logic;
      async_n  : IN    std_logic; 
      sync     : OUT   std_logic);
END COMPONENT;

ATTRIBUTE syn_keep : BOOLEAN;

SIGNAL tri_en       : std_logic;
SIGNAL mclk         : std_logic;
SIGNAL delay_en     : std_logic;  ATTRIBUTE syn_keep OF delay_en : SIGNAL IS true;
SIGNAL debug_clk_en : std_logic; 
SIGNAL core_reset   : std_logic; 
SIGNAL debug        : umbilical;
SIGNAL debugROM     : debugrom_port;
SIGNAL in_reg       : data_bus;
SIGNAL out_trap     : std_logic; 
SIGNAL rom_load     : std_logic; 

SIGNAL dram         : dram_port;
SIGNAL rom          : rom_port;
SIGNAL ioBus        : io_port;

BEGIN

mclk <= clk;

-----------------------------------------------------------------------
--  TRI_EN using a pad with in- and output buffer as delay element
-- 'Z'-states uBus for 20 ns to prevent cross-currents
-- To force the synthesizer to generate such logic, the pad is defined
-- as a tri-state pad (INOUT) that is continuously enabled.
-----------------------------------------------------------------------

delay_en <= '1';
delay    <= mclk AFTER 20 ns WHEN delay_en='1' ELSE 'Z';

tristate_holdoff: PROCESS (delay, mclk)
BEGIN
  IF  delay='1'  THEN
     tri_en <= '1';
  ELSIF  rising_edge(mclk)  THEN
     IF  clk_en='1'  THEN
        tri_en <= '0';
     END IF;
  END IF;
END PROCESS tristate_holdoff;

-----------------------------------------------------------------------
--  Centronics debug interface
-----------------------------------------------------------------------

debug.in_data <= data;

synch_init:    synchronise_n PORT MAP(reset, mclk, init_n,   debug.init   );
synch_in_str:  synchronise_n PORT MAP(reset, mclk, strobe_n, debug.in_str );
synch_in_atn:  synchronise   PORT MAP(reset, mclk, autofd,   debug.in_atn );
synch_out_ack: synchronise_n PORT MAP(reset, mclk, slctin_n, debug.out_ack);

busy    <=     debug.busy;     
ack_n   <= NOT debug.in_ack;
error_n <= NOT debug.out_atn;
slct    <=     debug.out_str;
pe      <=     debug.out_data;

debug_unit: debugger
PORT MAP(reset      => reset,
         clk        => mclk,
         clk_en     => clk_en,
         rom        => rom,
         ioBus      => ioBus,
         debug      => debug,
         debugROM   => debugROM,
         in_reg     => in_reg,
         core_reset => core_reset,
         out_trap   => out_trap,
         rom_load   => rom_load);

romwr   <= debugROM.write;
romaddr <= debugROM.addr;
romdin  <= debugROM.din;
debugROM.dout <= instruction;

-----------------------------------------------------------------------
--  Core
-----------------------------------------------------------------------

debug_clk_en <= clk_en AND NOT rom_load;

uCore: microcore
PORT MAP(reset   => core_reset,
         clk     => mclk,
         clk_en  => debug_clk_en,
         tri_en  => tri_en,
         int_in  => int_in,
         dram    => dram,
         rom     => rom,
         ioBus   => ioBus);

data_memory: data_ram PORT MAP (dram => dram);
--ramwr     <= dram.write;
--ramaddr   <= dram.addr;
--ramdin    <= dram.din;
--dram.dout <= ramdout;
dram.clk  <= clk;

--prog_memory: code_rom PORT MAP (rom => rom);
--romaddr   <= rom.addr;
--rom.data  <= instruction;

-----------------------------------------------------------------------
--  io interface
-----------------------------------------------------------------------

iosel       <= ioBus.sel;
ioBus.trap  <= iotrap AND ioBus.sel;
iowrite     <= ioBus.write;
ioaddr      <= ioBus.addr;
iodout      <= ioBus.dout;
ioBus.din   <= in_reg WHEN  ioBus.addr=DEBUG_REGISTER(data_addr_width-1 DOWNTO 0)  ELSE  iodin;

END technology;
