-- Do not use this file except in compliance with the License. You may
-- obtain a copy of the License at http://www.microcore.org/License/
-- Software distributed under the License is distributed on an "AS IS" basis,
-- WITHOUT WARRANTY OF ANY KIND, either express or implied.
-- See the License for the specific language governing rights and limitations
-- under the License.
--
-- The Original Code is: MicroCore INTERRUPT.VHD 1.10
--
-- The Initial Developer of the Original Code is Klaus.Schleisiek AT hamburg.de.
--
-- Introduction  see: information/uCore_Introduction.pdf
-- Tools         see: information/uCore_Implementation.pdf
-- Versions      see: uCore_Versions.txt
-- License       see: information/uCore_License.pdf
-- Conventions   see: information/uCore_Conventions.pdf
-- Contributors  see: information/uCore_Contributors.pdf
-- Philosophy    see: information/uCore_Philosophy.pdf
--
-- Interrupt signal preprocessing with FLAGS register
-- to signal interrupt sources and the IE register to
-- enable individual interrupts.

LIBRARY IEEE;
USE     IEEE.std_logic_1164.ALL;
USE     IEEE.std_logic_signed.ALL;
USE     work.functions.ALL;
USE     work.constants.ALL;

ENTITY interrupt IS
PORT (reset     : IN    std_logic; 
      clk       : IN    std_logic; 
      clk_en    : IN    std_logic; 
      int_in    : IN    int_bus;
      inst      : IN    inst_bus;
      stBus     : INOUT status_rec;
      int       : OUT   std_logic;
      flags     : OUT   data_bus);
END interrupt;

ARCHITECTURE rtl OF interrupt IS

SIGNAL synchron   : int_bus;
SIGNAL flags_i    : int_bus;
SIGNAL ie         : int_bus;

SIGNAL int_en     : std_logic;
SIGNAL int_i      : std_logic;

SIGNAL s_ie       : std_logic;
SIGNAL s_iis      : std_logic;

BEGIN

int   <= int_i;
flags <= slice('0', data_width-interrupts-1) & flags_i & int_i;

sync_in: PROCESS (reset, clk)
BEGIN
   IF  reset='1'  THEN
      synchron <= (OTHERS => '0');
      flags_i  <= (OTHERS => '0');
      ie       <= (OTHERS => '0');
   ELSIF  rising_edge(clk)  THEN
      synchron <= int_in;
      flags_i  <= synchron AND int_in;
      IF  inst=IE_OP  THEN
         IF  stBus.tos(0)='1'  THEN
               ie <= ie OR      stBus.tos(interrupts DOWNTO 1);
         ELSE  ie <= ie AND NOT stBus.tos(interrupts DOWNTO 1); 
         END IF;
      END IF;
   END IF;
END PROCESS sync_in;

int_en <= s_ie AND NOT s_iis;
int_i  <= int_en WHEN  (flags_i AND ie) /= 0  ELSE  '0';

------------------------------------------------------------------------------
-- status_register 
--
-- The process writes the STATUS' flags IE and IIS (Interrupt In Service). 
-- (a) reset = 1: s_ie = 0, s_iis = 0
-- (b) clk valid & int_en = 1: s_iis = int_i
--     int_i is set by process sync_in and means that synchronised interrupt
--     request was processed. 
-- (c) clk valid & status.write = 1:  s_ie = 0, s_iis = 0 from tos
-- 
------------------------------------------------------------------------------
status_register: PROCESS(clk, reset)
BEGIN
   IF  reset='1'  THEN
      s_ie  <= '0';
      s_iis <= '0';
   ELSIF  rising_edge(clk)  THEN
     IF  clk_en='1'  THEN
       IF  int_en='1'  THEN
          s_iis <= int_i;
       END IF;
       IF  stBus.write='1'  THEN
          s_ie  <= stBus.tos(s_ie_bit);
          s_iis <= stBus.tos(s_iis_bit) OR int_i;
       END IF; 
     END IF;
   END IF;
END PROCESS status_register;

stBus.flags(s_ie_bit)  <= s_ie;
stBus.flags(s_iis_bit) <= '0';

END rtl;
